function features = extractFeaturesFromHsIntervals(assigned_states, PCG)
%
% This function calculate 20 features based on the assigned_states by running "runSpringerSegmentationAlgorithm.m" function
%
% INPUTS:
% assigned_states: the array of state values assigned to the sound recording.
% PCG: resampled sound recording with 1000 Hz.
%
% OUTPUTS:
% features: the obtained 20 features for the current sound recording
%
%
% Written by: Chengyu Liu, January 22 2016
%             chengyu.liu@emory.edu
%
% Last modified by:
%
%
% $$$$$$ IMPORTANT
% Please note: the calculated 20 features are only some pilot features, some features maybe
% helpful for classifying normal/abnormal heart sounds, some maybe
% not. You need re-construct the features for a more accurate classification.


%% We just assume that the assigned_states cover at least 2 whole heart beat cycle
indx = find(abs(diff(assigned_states))>0); % find the locations with changed states

if assigned_states(1)>0   % for some recordings, there are state zeros at the beginning of assigned_states
    switch assigned_states(1)
        case 4
            K=1;
        case 3
            K=2;
        case 2
            K=3;
        case 1
            K=4;
    end
else
    switch assigned_states(indx(1)+1)
        case 4
            K=1;
        case 3
            K=2;
        case 2
            K=3;
        case 1
            K=0;
    end
    K=K+1;
end

indx2                = indx(K:end);
rem                  = mod(length(indx2),4);
indx2(end-rem+1:end) = [];
A                    = reshape(indx2,4,length(indx2)/4)'; % A is N*4 matrix, the 4 columns save the beginnings of S1, systole, S2 and diastole in the same heart cycle respectively

%% Feature calculation
m_RR        = round(mean(diff(A(:,1))));             % mean value of RR intervals
sd_RR       = round(std(diff(A(:,1))));              % standard deviation (SD) value of RR intervals
mean_IntS1  = round(mean(A(:,2)-A(:,1)));            % mean value of S1 intervals
sd_IntS1    = round(std(A(:,2)-A(:,1)));             % SD value of S1 intervals
mean_IntS2  = round(mean(A(:,4)-A(:,3)));            % mean value of S2 intervals
sd_IntS2    = round(std(A(:,4)-A(:,3)));             % SD value of S2 intervals
mean_IntSys = round(mean(A(:,3)-A(:,2)));            % mean value of systole intervals
sd_IntSys   = round(std(A(:,3)-A(:,2)));             % SD value of systole intervals
mean_IntDia = round(mean(A(2:end,1)-A(1:end-1,4)));  % mean value of diastole intervals
sd_IntDia   = round(std(A(2:end,1)-A(1:end-1,4)));   % SD value of diastole intervals

for i=1:size(A,1)-1
    R_SysRR(i)  = (A(i,3)-A(i,2))/(A(i+1,1)-A(i,1))*100;
    R_DiaRR(i)  = (A(i+1,1)-A(i,4))/(A(i+1,1)-A(i,1))*100;
    R_SysDia(i) = R_SysRR(i)/R_DiaRR(i)*100;
    
    P_S1(i)     = sum(abs(PCG(A(i,1):A(i,2))))/(A(i,2)-A(i,1));
    P_Sys(i)    = sum(abs(PCG(A(i,2):A(i,3))))/(A(i,3)-A(i,2));
    P_S2(i)     = sum(abs(PCG(A(i,3):A(i,4))))/(A(i,4)-A(i,3));
    P_Dia(i)    = sum(abs(PCG(A(i,4):A(i+1,1))))/(A(i+1,1)-A(i,4));
    if P_S1(i)>0
        P_SysS1(i) = P_Sys(i)/P_S1(i)*100;
    else
        P_SysS1(i) = 0;
    end
    if P_S2(i)>0
        P_DiaS2(i) = P_Dia(i)/P_S2(i)*100;
    else
        P_DiaS2(i) = 0;
    end
end

m_Ratio_SysRR   = mean(R_SysRR);  % mean value of the interval ratios between systole and RR in each heart beat
sd_Ratio_SysRR  = std(R_SysRR);   % SD value of the interval ratios between systole and RR in each heart beat
m_Ratio_DiaRR   = mean(R_DiaRR);  % mean value of the interval ratios between diastole and RR in each heart beat
sd_Ratio_DiaRR  = std(R_DiaRR);   % SD value of the interval ratios between diastole and RR in each heart beat
m_Ratio_SysDia  = mean(R_SysDia); % mean value of the interval ratios between systole and diastole in each heart beat
sd_Ratio_SysDia = std(R_SysDia);  % SD value of the interval ratios between systole and diastole in each heart beat

indx_sys = find(P_SysS1>0 & P_SysS1<100);   % avoid the flat line signal
if length(indx_sys)>1
    m_Amp_SysS1  = mean(P_SysS1(indx_sys)); % mean value of the mean absolute amplitude ratios between systole period and S1 period in each heart beat
    sd_Amp_SysS1 = std(P_SysS1(indx_sys));  % SD value of the mean absolute amplitude ratios between systole period and S1 period in each heart beat
else
    m_Amp_SysS1  = 0;
    sd_Amp_SysS1 = 0;
end
indx_dia = find(P_DiaS2>0 & P_DiaS2<100);
if length(indx_dia)>1
    m_Amp_DiaS2  = mean(P_DiaS2(indx_dia)); % mean value of the mean absolute amplitude ratios between diastole period and S2 period in each heart beat
    sd_Amp_DiaS2 = std(P_DiaS2(indx_dia));  % SD value of the mean absolute amplitude ratios between diastole period and S2 period in each heart beat
else
    m_Amp_DiaS2  = 0;
    sd_Amp_DiaS2 = 0;
end

features1 = [sd_RR mean_IntSys  m_Ratio_SysRR sd_Ratio_SysRR m_Ratio_DiaRR sd_Ratio_DiaRR  m_Amp_SysS1 sd_Amp_SysS1  sd_Amp_DiaS2];
rows=size(A,1);
signal=PCG;
% figure;
% plot(signal);
%% Features
% if  size(A,1)>=100
%        rows=90;
% else rows=size(A,1);
% end
y5=signal;
for i=1:rows-1
    start(i)=A(i,1);
    end1(i)=A(i+1,4);
    timeseg(i,:)=[start(i),end1(i)];
    A(i,5)=[A(i+1)];
end
A(:,5)=A(:,5)-ones(rows,1);
    for j=1:length(timeseg(:,1))
    M{j} = y5(timeseg(j,1):timeseg(j,2));
    end
   
    for k=1:length(M)
    [C1{1,k},L1{1,k}] = wavedec(M{1,k},6,'db2');
    D22{1,k} = wrcoef('d',C1{1,k},L1{1,k},'db2',2)';
%     windowsize=4;
    end
    anew=A-[(A(:,1)-1),(A(:,1)-1),(A(:,1)-1),(A(:,1)-1),(A(:,1)-1)];
%     anw=[A(2:end,1)-A(1:end-1,4)];
%     anw1=(anw-(A(2:end,1)-1));
%     anew=[an(1:end-1,:) anw];
      for kj=1:length(D22)
      
            S1_seg{1,kj}(1,:)=(D22{1,kj}(anew(kj,1):anew(kj,2)));
            Sys_seg{1,kj}(1,:)=(D22{1,kj}(anew(kj,2):anew(kj,3)));
            S2_seg{1,kj}(1,:)=(D22{1,kj}(anew(kj,3):anew(kj,4)));
            ds_seg{1,kj}(1,:)=(D22{1,kj}(anew(kj,4):anew(kj,5)));
         
      end
              
%       %     totalfeature1(:,:) =  [D22{1,k}];
%      numSamples(kj) = length(D22{1,kj}); 
%      frameWidth(kj) = floor(numSamples(kj)/windowsize);
%      trailingsamples(kj) = mod(numSamples(kj), windowsize);
%      sampleframes{1,kj} = reshape( D22{1,kj}(1:end-trailingsamples),frameWidth(kj),[])';

    for kjl=1:length(D22)
%       for ij=1:windowsize
       lpS1(1,kjl)=mean(lpc(S1_seg{1,kjl}));
       lpSys(1,kjl)=mean(lpc(Sys_seg{1,kjl}));
       lpS2(1,kjl)=mean(lpc(S2_seg{1,kjl}));
       lpds(1,kjl)=mean(lpc(ds_seg{1,kjl}));
       
       skewS1(1,kjl)=skewness(S1_seg{1,kjl}(1,:));
       skewSys(1,kjl)=skewness(Sys_seg{1,kjl}(1,:));
       skewS2(1,kjl)=skewness(S2_seg{1,kjl}(1,:));
       skewds(1,kjl)=skewness(ds_seg{1,kjl}(1,:));
       
       kurS1(1,kjl) = kurtosis(S1_seg{1,kjl}(1,:));
       kurSys(1,kjl) = kurtosis(Sys_seg{1,kjl}(1,:));
       kurS2(1,kjl) = kurtosis(S2_seg{1,kjl}(1,:));
       kurds(1,kjl) = kurtosis(ds_seg{1,kjl}(1,:));

       [PxxS1{1,kjl},f1{1,kjl}]=pwelch(S1_seg{1,kjl}(1,:),[],[],[]);
       PxxSys{1,kjl}=pwelch(Sys_seg{1,kjl}(1,:),[],[],[]);
       [PxxS2{1,kjl},f2{1,kjl}]=pwelch(S2_seg{1,kjl}(1,:),[],[],[]);
       [Pxxds{1,kjl},f3{1,kjl}]=pwelch(ds_seg{1,kjl}(1,:),[],[],[]);
%        Pxx4{1,kj}=pwelch(sampleframes{1,kj}(4,:),[],[],[]);
        xp1{1,kjl}=length(PxxS1{1,kjl});
        xp2{1,kjl}=length(PxxSys{1,kjl});
        xp3{1,kjl}=length(PxxS2{1,kjl});
        xp4{1,kjl}=length(Pxxds{1,kjl});
        
        power1(kjl) = trapz((1:xp1{1,kjl}), PxxS1{1,kjl}(1:xp1{1,kjl}));
        power2(kjl) = trapz((1:xp2{1,kjl}), PxxSys{1,kjl}(1:xp2{1,kjl}));
        power3(kjl) = trapz((1:xp3{1,kjl}), PxxS2{1,kjl}(1:xp3{1,kjl}));
        power4(kjl) = trapz((1:xp4{1,kjl}), Pxxds{1,kjl}(1:xp4{1,kjl}));
        
        ratio(kjl)=power1(kjl)/power3(kjl);
        MaxpS1(kjl)=max(PxxS1{1,kjl});
        MaxpS2(kjl)=max(PxxS2{1,kjl});
    end

    for kjl1=1:length(D22)-1
        for i2=1:length(PxxS1{1,kjl1})
        if PxxS1{1,kjl1}(i2)==MaxpS1(kjl1)
         dom_freq1(kjl1) = f1{1,kjl1}(i2);
%          max_ind = i2;
        end
        if PxxS2{1,kjl1}(i2)==MaxpS2(kjl1)
         dom_freq2(kjl1) = f2{1,kjl1}(i2);
%          max_ind = i2;
        end
        end
    end    

%%
for kj2=3:length(D22)
peaksys{1,kj2} = findpeaks(Sys_seg{1,kj2});
peakds{1,kj2} = findpeaks(ds_seg{1,kj2});
mnpsys(1,kj2)= mean(peaksys{1,kj2});
datads(1,kj2)= mean(peakds{1,kj2});
end
%%
domS1=mean(dom_freq1);
domS2=mean(dom_freq2);
stS1=std(dom_freq1);
stS2=std(dom_freq2);


total_power1=mean(power1);
total_power2=mean(power2);
total_power3=mean(power3);
total_power4=mean(power4);

st_power1=std(power1);
st_power2=std(power2);
st_power3=std(power3);
 st_power4=std(power4);
% 
total_lpS1=mean(lpS1);
total_lpSys=mean(lpSys);
total_lpS2=mean(lpS2);
total_lpds=mean(lpds);

std_lpS1=std(lpS1);
std_lpSys=std(lpSys);
std_lpS2=std(lpS2);
std_lpds=mean(lpds);

total_skewS1=mean(skewS1);
total_skewSys=mean(skewSys);
total_skewS2=mean(skewS2);
total_skew4=mean(skewds);

std_skewS1=std(skewS1);
std_skewSys=std(skewSys);
std_skewS2=std(skewS2);
std_skew4=mean(skewds);
% 
% % stskew=std(Skew1);
total_kS1=mean(kurS1);
total_kSys=mean(kurSys);
total_kS2=mean(kurS2);
total_kds=mean(kurds);

st_kS1=std(kurS1);
st_kSys=std(kurSys);
st_kS2=std(kurS2);
st_kds=std(kurds);

avrratio=mean(ratio);
stratio=std(ratio);

opsys=mean(mnpsys);
stsys=std(mnpsys);
opds=mean(datads);
stds=std(datads);
op=[opsys stsys opds stds];
% % power = ); 
  feature2=[total_lpS1 total_lpSys total_lpS2 total_lpds std_lpS1 std_lpSys std_lpS2 std_lpds domS1 domS2 stS1 stS2 total_power2 total_power3 total_power4 st_power1 st_power2 st_power4 total_skewS1 total_skewSys total_skewS2 total_skew4 std_skewS1 std_skewSys std_skewS2 std_skew4 total_kSys total_kS2 total_kds st_kS1 st_kSys st_kS2 st_kds avrratio];
% 
% % All_feature=[total_power1 total_power2 total_skew1 total_skew2 total_k1 total_k2];
% %All_feature=[total_power total_skew stskew total_k stk];
% % All_feature=[total_power total_skew  total_k];
% %% Feature Munia
% 
 features=[feature2 features1];

